"use strict";
/**
 * Render
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRenderFunctions = exports.renderContent = exports.renderItem = exports.render = void 0;
const contentType_js_1 = require("../utils/contentType/contentType.js");
const action_js_1 = require("../utils/action/action.js");
const filter_js_1 = require("../utils/filter/filter.js");
const link_js_1 = require("../utils/link/link.js");
const json_js_1 = require("../utils/json/json.js");
const string_js_1 = require("../utils/string/string.js");
const array_js_1 = require("../utils/array/array.js");
const object_js_1 = require("../utils/object/object.js");
const number_js_1 = require("../utils/number/number.js");
const function_js_1 = require("../utils/function/function.js");
const shortcode_js_1 = require("../utils/shortcode/shortcode.js");
const tag_js_1 = require("../utils/tag/tag.js");
const config_js_1 = require("../config/config.js");
const Container_js_1 = require("../layouts/Container/Container.js");
const Column_js_1 = require("../layouts/Column/Column.js");
const Form_js_1 = require("../objects/Form/Form.js");
const Field_js_1 = require("../objects/Field/Field.js");
const RichText_js_1 = require("../text/RichText/RichText.js");
/**
 * Store slug data
 *
 * @private
 * @type {RenderSlugs}
 */
const slugs = {};
/**
 * Normalize meta properties into one object
 *
 * @private
 * @param {RenderMetaArgs} args
 * @return {RenderMetaReturn}
 */
const getMeta = (args) => {
    const meta = {
        title: '',
        description: '',
        url: '',
        image: '',
        canonical: '',
        prev: '',
        next: '',
        noIndex: false,
        isIndex: false
    };
    if (!(0, object_js_1.isObjectStrict)(args)) {
        return meta;
    }
    if ((0, object_js_1.isObjectStrict)(args.meta)) {
        return Object.assign(meta, args.meta);
    }
    if ((0, string_js_1.isStringStrict)(args.metaTitle)) {
        meta.title = args.metaTitle;
    }
    if ((0, string_js_1.isStringStrict)(args.metaDescription)) {
        meta.description = args.metaDescription;
    }
    if ((0, string_js_1.isStringStrict)(args?.metaImage?.url)) {
        meta.image = args.metaImage.url;
    }
    return meta;
};
/**
 * Add pagination data to meta object
 *
 * @private
 * @param {PaginationData} args
 * @return {void}
 */
const setPaginationMeta = (args, slugArgs, meta) => {
    const { current = 0, currentFilters, prevFilters, nextFilters } = args;
    slugArgs.page = current > 1 ? current : 0;
    const s = (0, link_js_1.getSlug)(slugArgs, true);
    meta.canonical = `${(0, link_js_1.getPermalink)(s.slug, current === 1)}${(0, string_js_1.isString)(currentFilters) ? currentFilters : ''}`;
    if ((0, string_js_1.isStringStrict)(args.title)) {
        meta.paginationTitle = args.title;
    }
    if ((0, number_js_1.isNumber)(args.prev)) {
        slugArgs.page = args.prev > 1 ? args.prev : 0;
        const p = (0, link_js_1.getSlug)(slugArgs, true);
        meta.prev = `${(0, link_js_1.getPermalink)(p.slug, args.prev === 1)}${(0, string_js_1.isString)(prevFilters) ? prevFilters : ''}`;
    }
    if ((0, number_js_1.isNumber)(args.next) && args.next > 1) {
        slugArgs.page = args.next;
        const n = (0, link_js_1.getSlug)(slugArgs, true);
        meta.next = `${(0, link_js_1.getPermalink)(n.slug, false)}${(0, string_js_1.isString)(nextFilters) ? nextFilters : ''}`;
    }
};
/**
 * Get content and templates in expected format to map
 *
 * @private
 * @param {RenderItem[]} content
 * @param {RenderItem[]} [_templates]
 * @return {RenderContentTemplate}
 */
const getContentTemplate = (content, _templates = []) => {
    /* Content must be array */
    if (!(0, array_js_1.isArrayStrict)(content)) {
        return {
            content: [],
            templates: _templates
        };
    }
    /* One level loop */
    const _content = content.map((c) => {
        /* Check if template */
        if ((0, tag_js_1.tagExists)(c, 'template')) {
            _templates.push(structuredClone(c));
            /* Replace template with template break */
            return {
                metadata: {
                    tags: [
                        {
                            id: 'templateBreak',
                            name: ''
                        }
                    ]
                }
            };
        }
        /* Output */
        return c;
    });
    /* Output */
    return {
        content: _content,
        templates: _templates
    };
};
/**
 * Map out content slots to templates for contentTemplate
 *
 * @private
 * @param {RenderItem[]} templates
 * @param {RenderItem[]} [content]
 * @return {RenderItem[]}
 */
const mapContentTemplate = (templates, content = []) => {
    /* Templates must be arrays */
    if (!(0, array_js_1.isArrayStrict)(templates)) {
        return templates;
    }
    /* Recurse templates */
    const lastTemplateIndex = templates.length - 1;
    templates.forEach((t, i) => {
        /* Remove template break */
        if (content[0] !== undefined && (0, tag_js_1.tagExists)(content[0], 'templateBreak') && content.length >= 1) {
            content.shift();
        }
        /* Check if optional - remove if no content */
        const isOptional = (0, tag_js_1.tagExists)(t, 'templateOptional');
        if (isOptional && i === lastTemplateIndex && content.length === 0) {
            templates.pop();
            return;
        }
        /* Check if slot */
        const isSlot = (0, tag_js_1.tagExists)(t, 'templateSlot');
        /* Check for repeatable template item */
        const children = t.content;
        if ((0, array_js_1.isArrayStrict)(children) && !isSlot) {
            let repeat;
            const repeatIndex = children.findIndex((c) => {
                const isRepeat = (0, tag_js_1.tagExists)(c, 'templateRepeat');
                if (isRepeat) {
                    repeat = c;
                }
                return isRepeat;
            });
            if (repeatIndex !== -1 && repeat !== undefined) {
                let breakIndex = content.findIndex((c) => {
                    return (0, tag_js_1.tagExists)(c, 'templateBreak');
                });
                breakIndex = breakIndex === -1 ? content.length : breakIndex;
                let insertIndex = repeatIndex;
                for (let j = insertIndex; j < breakIndex - 1; j += 1) {
                    children.splice(insertIndex, 0, structuredClone(repeat));
                    insertIndex = j;
                }
            }
        }
        /* Replace slot with content */
        if (isSlot && content.length >= 1) {
            const fill = content.shift();
            if (fill !== undefined) {
                templates[i] = fill;
            }
            return;
        }
        /* Recurse children */
        if ((0, array_js_1.isArray)(children) && templates[i] !== undefined) {
            templates[i].content = mapContentTemplate(children, content);
        }
    });
    /* Output */
    return templates;
};
/**
 * Get config and formation render functions
 *
 * @return {RenderFunctions}
 */
const getRenderFunctions = () => {
    const renderFunctions = { ...config_js_1.config.renderFunctions };
    if (renderFunctions.container === undefined) {
        renderFunctions.container = Container_js_1.Container;
    }
    if (renderFunctions.column === undefined) {
        renderFunctions.column = Column_js_1.Column;
    }
    if (renderFunctions.form === undefined) {
        renderFunctions.form = Form_js_1.Form;
    }
    if (renderFunctions.field === undefined) {
        renderFunctions.field = Field_js_1.Field;
    }
    if (renderFunctions.richText === undefined) {
        renderFunctions.richText = RichText_js_1.RichText;
    }
    return renderFunctions;
};
exports.getRenderFunctions = getRenderFunctions;
/**
 * Recurse and output nested content
 *
 * @param {RenderContentArgs} args
 * @param {HtmlString} [_output]
 * @return {Promise<string>}
 */
const renderContent = async (args, _output = { html: '' }) => {
    if (!(0, object_js_1.isObjectStrict)(args)) {
        return _output.html;
    }
    const { content = [], serverlessData, pageData, pageContains = [], pageHeadings = [], navigations = {}, renderFunctions = {} } = args;
    let { parents = [], headingsIndex = 0, depth = 0 } = args;
    /* Content must be array */
    if (!(0, array_js_1.isArrayStrict)(content)) {
        return _output.html;
    }
    /* Loop */
    for (const item of content) {
        /* Item must be object */
        if (!(0, object_js_1.isObjectStrict)(item)) {
            continue;
        }
        /* Render props */
        const props = structuredClone(item);
        const renderType = (0, string_js_1.isString)(props.renderType) ? props.renderType : '';
        const contentType = (0, string_js_1.isString)(props.contentType) ? props.contentType : ''; // NORMALIZEEE???
        /* Check for nested content */
        let children = props.content;
        /* Map out content to template */
        if ((0, contentType_js_1.normalizeContentType)(contentType) === 'contentTemplate') {
            const contentTemplate = getContentTemplate((0, array_js_1.isArray)(props.content) ? props.content : []);
            const templates = mapContentTemplate(contentTemplate.templates, contentTemplate.content);
            children = templates;
        }
        /* Children array check */
        let childrenArr;
        if ((0, array_js_1.isArrayStrict)(children) && renderType !== 'richText') {
            childrenArr = children;
        }
        /* Render output */
        let renderObj = {
            start: '',
            end: ''
        };
        let filterType = '';
        let filterArgs = {};
        const renderFunction = renderFunctions[renderType];
        if ((0, function_js_1.isFunction)(renderFunction)) {
            const renderArgs = {
                args: props,
                parents,
                pageData,
                pageContains,
                navigations,
                serverlessData
            };
            if (childrenArr !== undefined) {
                renderArgs.children = childrenArr;
            }
            if (renderType === 'richText') {
                renderArgs.headings = pageHeadings[headingsIndex];
            }
            const renderOutput = await renderFunction(renderArgs);
            if ((0, string_js_1.isString)(renderOutput)) {
                renderObj.start = renderOutput;
            }
            else {
                renderObj = renderOutput;
            }
            pageContains.push(renderType);
            filterType = renderType;
            filterArgs = {
                ...props,
                content: undefined
            };
        }
        let start = renderObj.start;
        let end = renderObj.end;
        /* Filter start and end output */
        const renderContentFilterArgs = {
            renderType: filterType,
            args: filterArgs
        };
        if (start !== '' && end === '') {
            start = await (0, filter_js_1.applyFilters)('renderContent', start, renderContentFilterArgs);
        }
        else {
            start = await (0, filter_js_1.applyFilters)('renderContentStart', start, renderContentFilterArgs);
            end = await (0, filter_js_1.applyFilters)('renderContentEnd', end, renderContentFilterArgs);
        }
        /* Add to output object */
        _output.html += start;
        /* Recurse through children */
        if (childrenArr !== undefined) {
            const parentsCopy = [...parents];
            parentsCopy.unshift({
                renderType,
                args: {
                    ...props,
                    content: undefined,
                    parents: undefined
                }
            });
            await renderContent({
                content: childrenArr,
                serverlessData,
                parents: parentsCopy,
                pageData,
                pageContains,
                pageHeadings,
                navigations,
                renderFunctions,
                headingsIndex,
                depth: depth += 1
            }, _output);
        }
        _output.html += end;
        /* Additional rich text areas */
        if (renderType === 'content' && depth === 0) {
            headingsIndex = pageHeadings.push([]);
        }
    }
    /* Output */
    return _output.html;
};
exports.renderContent = renderContent;
/**
 * Output single post or page
 *
 * @param {RenderItemArgs} args
 * @return {RenderItemReturn}
 */
const renderItem = async (args) => {
    if (!(0, object_js_1.isObjectStrict)(args)) {
        return {};
    }
    const { item, contentType = 'page', serverlessData, renderFunctions } = args;
    /* Item must be object */
    if (!(0, object_js_1.isObjectStrict)(item)) {
        return {};
    }
    /* Taxonomy not page */
    if (contentType === 'taxonomy' && item.isPage !== true) {
        return {};
    }
    /* Item id required */
    const id = item.id;
    if (!(0, string_js_1.isStringStrict)(id)) {
        return {};
    }
    /* Slug required */
    if (!(0, string_js_1.isStringStrict)(item.slug)) {
        return {};
    }
    /* Serverless render check */
    let serverlessRender = false;
    /* Store components contained in page  */
    const pageContains = [];
    /* Store rich text headings in page */
    const pageHeadings = [[]];
    /* Start action */
    const renderItemStartArgs = {
        id,
        pageData: item,
        contentType,
        pageContains,
        pageHeadings,
        serverlessData
    };
    await (0, action_js_1.doActions)('renderItemStart', renderItemStartArgs);
    /* Reset script and style files */
    config_js_1.config.scripts.deps.clear();
    config_js_1.config.scripts.item.clear();
    config_js_1.config.styles.deps.clear();
    config_js_1.config.styles.item.clear();
    /* Meta */
    const title = item.title;
    const meta = getMeta(item);
    if (!(0, string_js_1.isStringStrict)(meta.title)) {
        meta.title = title;
    }
    /* Term taxonomy */
    const taxonomy = item.taxonomy;
    /* Permalink */
    const slugArgs = {
        id,
        contentType,
        slug: item.slug,
        returnParents: true,
        pageData: item,
        page: 0
    };
    const s = (0, link_js_1.getSlug)(slugArgs, true);
    const slug = s.slug;
    const permalink = (0, link_js_1.getPermalink)(slug);
    const parents = s.parents;
    item.basePermalink = (0, link_js_1.getPermalink)(slug);
    meta.url = permalink;
    meta.canonical = permalink;
    /* Add to data by slugs store */
    let formattedSlug = slug !== 'index' && slug !== '' ? `/${slug}/` : '/';
    if (slug.includes('.html')) {
        formattedSlug = slug;
    }
    slugs[formattedSlug] = {
        contentType,
        id
    };
    /* Check if index */
    const index = item.slug === 'index';
    meta.isIndex = index;
    /* Navigations */
    let navigations = {};
    if ((0, function_js_1.isFunction)(config_js_1.config.renderNavigations)) {
        let currentType = contentType;
        if ((0, object_js_1.isObjectStrict)(taxonomy)) {
            currentType = (0, string_js_1.isStringStrict)(taxonomy.contentType) ? taxonomy.contentType : contentType;
        }
        navigations = await config_js_1.config.renderNavigations({
            navigations: config_js_1.config.navigation,
            items: config_js_1.config.navigationItem,
            currentLink: permalink,
            currentType,
            title,
            parents
        });
    }
    /* Serverless data */
    let itemServerlessData;
    if ((0, object_js_1.isObjectStrict)(serverlessData)) {
        const serverlessPath = serverlessData.path !== undefined ? serverlessData.path : '';
        if (serverlessPath === formattedSlug && serverlessData.query !== undefined) {
            itemServerlessData = serverlessData;
        }
        else { // Avoid re-rendering non dynamic pages
            return {
                serverlessRender: false
            };
        }
    }
    /* Page data (props) for layout and actions */
    const pageData = { ...item };
    pageData.id = id;
    pageData.parents = parents;
    pageData.content = undefined;
    /* Content loop */
    let contentOutput = '';
    const contentData = item.content;
    if ((0, array_js_1.isArrayStrict)(contentData)) {
        contentOutput = await renderContent({
            content: contentData,
            serverlessData: itemServerlessData,
            parents: [],
            pageData,
            pageContains,
            pageHeadings,
            navigations,
            renderFunctions
        });
    }
    contentOutput = await (0, shortcode_js_1.doShortcodes)(contentOutput);
    /* Pagination variables for meta object */
    if ((0, object_js_1.isObjectStrict)(pageData.pagination)) {
        setPaginationMeta(pageData.pagination, slugArgs, meta);
        serverlessRender = true;
    }
    /* Output */
    let layoutOutput = '';
    if ((0, function_js_1.isFunction)(config_js_1.config.renderLayout)) {
        const layoutArgs = {
            id,
            meta,
            navigations,
            contentType,
            content: contentOutput,
            slug: formattedSlug,
            pageContains,
            pageHeadings,
            pageData,
            serverlessData
        };
        layoutOutput = await config_js_1.config.renderLayout(layoutArgs);
    }
    const renderItemFilterArgs = {
        id,
        contentType,
        slug: formattedSlug,
        output: layoutOutput,
        pageData,
        pageContains,
        pageHeadings,
        serverlessData
    };
    layoutOutput = await (0, filter_js_1.applyFilters)('renderItem', layoutOutput, renderItemFilterArgs);
    /* End action */
    const renderItemEndArgs = {
        id,
        contentType,
        slug: formattedSlug,
        output: layoutOutput,
        pageData,
        pageContains,
        pageHeadings,
        serverlessData
    };
    await (0, action_js_1.doActions)('renderItemEnd', renderItemEndArgs);
    /* Output */
    return {
        serverlessRender,
        pageData,
        data: {
            slug: formattedSlug,
            output: layoutOutput
        }
    };
};
exports.renderItem = renderItem;
/**
 * Loop through all content types to output pages and posts
 *
 * @param {RenderArgs} args
 * @return {Promise<RenderReturn[]|RenderReturn>}
 */
const render = async (args) => {
    /* Fallback output */
    const fallback = [{
            slug: '',
            output: ''
        }];
    /* Props must be object */
    if (!(0, object_js_1.isObjectStrict)(args)) {
        return fallback;
    }
    const { allData, serverlessData, previewData } = args;
    /* Serverless */
    const isServerless = serverlessData !== undefined;
    const isPreview = previewData !== undefined;
    /* Start action */
    await (0, action_js_1.doActions)('renderStart', args);
    /* Reset script and style directories */
    config_js_1.config.scripts.deps.clear();
    config_js_1.config.scripts.item.clear();
    config_js_1.config.styles.deps.clear();
    config_js_1.config.styles.item.clear();
    /* Data */
    if (!(0, object_js_1.isObjectStrict)(allData)) {
        return fallback;
    }
    const { navigation, navigationItem, redirect, content } = allData;
    /* Store navigations and items */
    if (navigation !== undefined) {
        config_js_1.config.navigation = navigation;
    }
    if (navigationItem !== undefined) {
        config_js_1.config.navigationItem = navigationItem;
    }
    /* Add formation functions */
    const renderFunctions = getRenderFunctions();
    /* Store content data */
    const data = [];
    /* Loop through pages first to set meta info */
    if (!isServerless) {
        for (const item of content.page) {
            /* Item must be object */
            if (!(0, object_js_1.isObjectStrict)(item)) {
                continue;
            }
            /* Parent and archive */
            const { parent, archive } = item;
            /* Id */
            const id = (0, string_js_1.isStringStrict)(item.id) ? item.id : '';
            /* Archive */
            const archiveType = (0, contentType_js_1.normalizeContentType)(archive);
            if ((0, string_js_1.isStringStrict)(archiveType) && id !== '') {
                const archiveSlug = item.slug;
                const archiveTitle = item.title;
                const archiveObj = (0, object_js_1.isObjectStrict)(config_js_1.config.archiveMeta[archiveType]) ? config_js_1.config.archiveMeta[archiveType] : {};
                config_js_1.config.archiveMeta[archiveType] = {
                    id,
                    slug: archiveSlug,
                    title: archiveTitle,
                    ...archiveObj
                };
            }
            /* Parent */
            if ((0, object_js_1.isObjectStrict)(parent) && id !== '') {
                const parentSlug = parent.slug;
                const parentTitle = parent.title;
                const parentId = parent.id;
                if ((0, string_js_1.isStringStrict)(parentSlug) && (0, string_js_1.isStringStrict)(parentTitle) && (0, string_js_1.isStringStrict)(parentId)) {
                    config_js_1.config.parents[id] = {
                        id: parentId,
                        slug: parentSlug,
                        title: parentTitle,
                        contentType: 'page'
                    };
                }
            }
        }
        /* Redirect */
        if ((0, array_js_1.isArrayStrict)(redirect)) {
            redirect.forEach((r) => {
                if (!(0, object_js_1.isObjectStrict)(r)) {
                    return;
                }
                const { redirect: rr = [] } = r;
                if (!(0, array_js_1.isArrayStrict)(rr)) {
                    return;
                }
                config_js_1.config.redirects.data = config_js_1.config.redirects.data.concat(rr);
            });
        }
    }
    else {
        const parentsData = await (0, json_js_1.getJsonFile)('parents');
        const archiveMetaData = await (0, json_js_1.getJsonFile)('archiveMeta');
        const formMetaData = await (0, json_js_1.getJsonFile)('formMeta');
        const navigationsData = await (0, json_js_1.getJsonFile)('navigations');
        const navigationItemsData = await (0, json_js_1.getJsonFile)('navigationItems');
        if (parentsData !== undefined) {
            Object.entries(parentsData).forEach(([key, value]) => {
                config_js_1.config.parents[key] = value;
            });
        }
        if (archiveMetaData !== undefined) {
            config_js_1.config.archiveMeta = archiveMetaData;
        }
        if (formMetaData !== undefined) {
            config_js_1.config.formMeta = formMetaData;
        }
        if (navigationsData !== undefined) {
            config_js_1.config.navigation = navigationsData;
        }
        if (navigationItemsData !== undefined) {
            config_js_1.config.navigationItem = navigationItemsData;
        }
    }
    /* Empty serverless reload */
    Object.keys(config_js_1.config.serverlessRoutes).forEach((r) => {
        config_js_1.config.serverlessRoutes[r] = [];
    });
    /* Loop through all content types */
    for (const [contentType, contentItems] of Object.entries(content)) {
        if (!(0, array_js_1.isArrayStrict)(contentItems)) {
            continue;
        }
        for (const contentItem of contentItems) {
            const item = await renderItem({
                item: contentItem,
                contentType,
                serverlessData,
                renderFunctions
            });
            const { serverlessRender = false, data: itemData } = item;
            if (itemData === undefined) {
                continue;
            }
            data.push(itemData);
            if (serverlessRender && !isServerless) {
                config_js_1.config.serverlessRoutes.reload.push({
                    path: itemData.slug.replace(/^\/|\/$/gm, '')
                });
            }
        }
    }
    /* Store files data */
    if (!isServerless && !isPreview) {
        config_js_1.config.storeFiles.slugs.data = JSON.stringify(slugs);
        config_js_1.config.storeFiles.parents.data = JSON.stringify(config_js_1.config.parents);
        config_js_1.config.storeFiles.navigations.data = JSON.stringify(config_js_1.config.navigation);
        config_js_1.config.storeFiles.navigationItems.data = JSON.stringify(config_js_1.config.navigationItem);
        if (config_js_1.config.storeFiles?.archiveMeta?.data !== undefined) {
            config_js_1.config.storeFiles.archiveMeta.data = JSON.stringify(config_js_1.config.archiveMeta);
        }
        if (config_js_1.config.storeFiles?.formMeta?.data !== undefined) {
            config_js_1.config.storeFiles.formMeta.data = JSON.stringify(config_js_1.config.formMeta);
        }
    }
    /* Output */
    const [outputItem] = data;
    const output = (isServerless || isPreview) && outputItem !== undefined ? outputItem : data;
    await (0, action_js_1.doActions)('renderEnd', { ...args, data: output });
    return output;
};
exports.render = render;
