"use strict";
/**
 * Utils - Archive
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getArchiveLink = exports.getTaxonomyInfo = exports.getArchiveInfo = void 0;
const config_js_1 = require("../../config/config.js");
const object_js_1 = require("../object/object.js");
const string_js_1 = require("../string/string.js");
const boolean_js_1 = require("../boolean/boolean.js");
const contentType_js_1 = require("../contentType/contentType.js");
const link_js_1 = require("../link/link.js");
/**
 * Get archive id, slug and title
 *
 * @param {string} contentType
 * @return {ArchiveInfo}
 */
const getArchiveInfo = (contentType = '') => {
    const value = {
        id: '',
        slug: '',
        title: ''
    };
    contentType = (0, contentType_js_1.normalizeContentType)(contentType);
    if (contentType === 'page') {
        return value;
    }
    const info = config_js_1.config.archiveMeta[contentType];
    if (!(0, object_js_1.isObjectStrict)(info)) {
        return value;
    }
    const { id, slug, title } = info;
    value.id = (0, string_js_1.isStringStrict)(id) ? id : '';
    value.slug = (0, string_js_1.isStringStrict)(slug) ? slug : '';
    value.title = (0, string_js_1.isStringStrict)(title) ? title : '';
    return value;
};
exports.getArchiveInfo = getArchiveInfo;
/**
 * Get taxonomy attributes
 *
 * @param {string} contentType
 * @param {RenderItem} pageData
 * @return {Taxonomy}
 */
const getTaxonomyInfo = (contentType, pageData) => {
    const value = {
        id: '',
        slug: '',
        title: '',
        contentType: '',
        useContentTypeSlug: true,
        isPage: false
    };
    const taxObj = contentType === 'taxonomy' ? pageData : pageData?.taxonomy;
    if (!(0, object_js_1.isObjectStrict)(taxObj)) {
        return value;
    }
    const { id, slug, title, contentType: type, useContentTypeSlug, isPage } = taxObj;
    value.id = (0, string_js_1.isStringStrict)(id) ? id : '';
    value.slug = (0, string_js_1.isStringStrict)(slug) ? slug : '';
    value.title = (0, string_js_1.isStringStrict)(title) ? title : '';
    value.contentType = (0, string_js_1.isStringStrict)(type) ? (0, contentType_js_1.normalizeContentType)(type) : '';
    value.useContentTypeSlug = (0, boolean_js_1.isBoolean)(useContentTypeSlug) ? useContentTypeSlug : true;
    value.isPage = (0, boolean_js_1.isBoolean)(isPage) ? isPage : false;
    return value;
};
exports.getTaxonomyInfo = getTaxonomyInfo;
/**
 * Get archive link by content type or taxonomy
 *
 * @param {string} contentType
 * @param {RenderItem} [pageData]
 * @return {ArchiveLinkReturn}
 */
const getArchiveLink = (contentType = '', pageData) => {
    /* Defaults */
    let title = '';
    let slug;
    /* Normalize */
    contentType = (0, contentType_js_1.normalizeContentType)(contentType);
    /* Taxonomy */
    const taxonomyInfo = getTaxonomyInfo(contentType, pageData);
    const { title: taxonomyTitle, isPage: taxonomyIsPage, contentType: taxonomyType, useContentTypeSlug: taxonomyUseTypeSlug } = taxonomyInfo;
    if (contentType === 'term' && taxonomyIsPage) {
        slug = (0, link_js_1.getSlug)({
            slug: '',
            contentType,
            pageData
        }, false);
        title = taxonomyTitle;
    }
    const useArchiveType = taxonomyUseTypeSlug && taxonomyType !== '';
    if (contentType === 'taxonomy' && useArchiveType) {
        contentType = taxonomyType;
    }
    if (contentType === 'term' && !taxonomyIsPage && useArchiveType) {
        contentType = taxonomyType;
    }
    /* Archive */
    const archiveInfo = getArchiveInfo(contentType);
    const { id: archiveId, slug: archiveSlug, title: archiveTitle } = archiveInfo;
    if (archiveSlug !== '' && archiveId !== '') {
        slug = (0, link_js_1.getSlug)({
            id: archiveId,
            slug: archiveSlug,
            contentType: 'page'
        }, false);
        const plural = config_js_1.config?.archiveMeta?.[contentType]?.plural;
        title = (0, string_js_1.isStringStrict)(plural) ? plural : archiveTitle;
    }
    /* Output */
    return {
        title,
        link: (0, string_js_1.isStringStrict)(slug) ? (0, link_js_1.getPermalink)(slug) : ''
    };
};
exports.getArchiveLink = getArchiveLink;
