"use strict";
/**
 * Utils - Contentful Data
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllContentfulData = exports.getContentfulData = void 0;
const contentful_resolve_response_1 = __importDefault(require("contentful-resolve-response"));
const contentfulDataNormal_js_1 = require("./contentfulDataNormal.js");
const filter_js_1 = require("../filter/filter.js");
const object_js_1 = require("../object/object.js");
const string_js_1 = require("../string/string.js");
const function_js_1 = require("../function/function.js");
const array_js_1 = require("../array/array.js");
const print_js_1 = require("../print/print.js");
const config_js_1 = require("../../config/config.js");
const json_js_1 = require("../json/json.js");
/**
 * Fetch data from contentful cms or cache
 *
 * @param {string} key
 * @param {ContentfulDataParams} params
 * @return {Promise<ContentfulDataReturn>}
 */
const getContentfulData = async (key = '', params = {}) => {
    try {
        /* Key required for cache */
        if (!(0, string_js_1.isStringStrict)(key)) {
            throw new Error('No key');
        }
        /* Check cache */
        if (config_js_1.config.env.cache) {
            let cacheData = {};
            const cacheDataFilterArgs = {
                key,
                type: 'get'
            };
            cacheData = await (0, filter_js_1.applyFilters)('cacheData', cacheData, cacheDataFilterArgs);
            if ((0, object_js_1.isObject)(cacheData)) {
                return structuredClone(cacheData);
            }
        }
        /* Credentials */
        const { space, devCredential, prodCredential, devHost, prodHost } = config_js_1.config.cms;
        let accessToken = devCredential;
        let host = devHost;
        if (config_js_1.config.env.prod) {
            accessToken = prodCredential;
            host = prodHost;
        }
        if (!(0, string_js_1.isStringStrict)(space) || !(0, string_js_1.isStringStrict)(accessToken) || !(0, string_js_1.isStringStrict)(host)) {
            throw new Error('No credentials');
        }
        /* Params */
        let url = `https://${host}/spaces/${space}/environments/master/entries?access_token=${accessToken}`;
        for (const [key, value] of Object.entries(params)) {
            url += `&${key}=${value.toString()}`;
        }
        /* Check and transform data */
        const resp = await fetch(url);
        const data = await resp.json();
        if (!(0, object_js_1.isObjectStrict)(data)) {
            throw new Error('No data');
        }
        if (data.items === undefined) {
            throw new Error('No items');
        }
        data.items = (0, contentful_resolve_response_1.default)(data);
        const newItems = (0, contentfulDataNormal_js_1.normalizeContentfulData)(data.items !== undefined ? data.items : []);
        const newData = {
            items: newItems
        };
        /* Store in cache */
        if (config_js_1.config.env.cache) {
            const cacheDataFilterArgs = {
                key,
                type: 'set',
                data
            };
            await (0, filter_js_1.applyFilters)('cacheData', newData, cacheDataFilterArgs);
        }
        /* Output */
        return newData;
    }
    catch (error) {
        (0, print_js_1.print)('[SSF] Error fetching Contentful data', error);
        /* Store in cache (avoid extra calls when no result) */
        if (config_js_1.config.env.cache) {
            const cacheDataFilterArgs = {
                key,
                type: 'set',
                data: {}
            };
            await (0, filter_js_1.applyFilters)('cacheData', {}, cacheDataFilterArgs);
        }
        /* Output */
        return {};
    }
};
exports.getContentfulData = getContentfulData;
/**
 * Fetch data from all content types or single entry if serverless
 *
 * @param {AllContentfulDataArgs} args
 * @return {Promise<RenderAllData|undefined>}
 */
const getAllContentfulData = async (args = {}) => {
    const { serverlessData, previewData, filterData, filterAllData } = args;
    try {
        /* Store all data */
        let allData = {
            navigation: [],
            navigationItem: [],
            redirect: [],
            content: {
                page: []
            }
        };
        /* Get single entry data if serverless or preview data */
        let isEntry = false;
        if (serverlessData !== undefined || previewData !== undefined) {
            let contentType = '';
            let id = '';
            if (serverlessData !== undefined) {
                const slugsData = await (0, json_js_1.getJsonFile)('slugs');
                const path = serverlessData.path;
                if (slugsData !== undefined) {
                    const item = slugsData[path];
                    if ((0, object_js_1.isObjectStrict)(item)) {
                        id = (0, string_js_1.isStringStrict)(item.id) ? item.id : '';
                        contentType = (0, string_js_1.isStringStrict)(item.contentType) ? item.contentType : '';
                    }
                }
            }
            if (previewData !== undefined) {
                id = previewData.id;
                contentType = previewData.contentType;
            }
            if (id !== '') {
                const key = `serverless_${id}_${contentType}`;
                const params = {
                    'sys.id': id,
                    include: 10
                };
                const data = await getContentfulData(key, params);
                if ((0, array_js_1.isArray)(data.items)) {
                    isEntry = true;
                    allData.content[contentType] = data.items;
                }
            }
        }
        /* Get partial data - not serverless */
        if (serverlessData === undefined || !isEntry) {
            const partial = config_js_1.config.partialTypes;
            for (const contentType of partial) {
                allData[contentType] = [];
                const key = `all_${contentType}`;
                const params = {
                    content_type: contentType
                };
                let data = await getContentfulData(key, params);
                if ((0, function_js_1.isFunction)(filterData)) {
                    data = filterData(data, serverlessData, previewData);
                }
                if ((0, array_js_1.isArray)(data.items)) {
                    allData[contentType] = data.items;
                }
            }
        }
        /* Get whole data (for page generation) - not serverless or preview */
        if ((serverlessData === undefined && previewData === undefined) || !isEntry) {
            const whole = config_js_1.config.wholeTypes;
            for (const contentType of whole) {
                allData.content[contentType] = [];
                const key = `all_${contentType}`;
                const params = {
                    content_type: contentType,
                    include: 10
                };
                let data = await getContentfulData(key, params);
                if ((0, function_js_1.isFunction)(filterData)) {
                    data = filterData(data, serverlessData, previewData);
                }
                if ((0, array_js_1.isArray)(data.items)) {
                    allData.content[contentType] = data.items;
                }
            }
        }
        /* Filter all data */
        if ((0, function_js_1.isFunction)(filterAllData)) {
            allData = filterAllData(allData, serverlessData, previewData);
        }
        /* Output */
        return allData;
    }
    catch (error) {
        (0, print_js_1.print)('[SSF] Error getting all Contentful data', error);
    }
};
exports.getAllContentfulData = getAllContentfulData;
