"use strict";
/**
 * Utils - File
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.createStoreFiles = exports.createRedirectsFile = exports.createServerlessFiles = void 0;
const promises_1 = require("node:fs/promises");
const node_path_1 = require("node:path");
const path_js_1 = require("../path/path.js");
const object_js_1 = require("../object/object.js");
const array_js_1 = require("../array/array.js");
const string_js_1 = require("../string/string.js");
const print_js_1 = require("../print/print.js");
const config_js_1 = require("../../config/config.js");
const dataSource_js_1 = require("../dataSource/dataSource.js");
/**
 * Minify template strings
 *
 * @param {string} str
 * @return {string}
 */
const minify = (str) => {
    return str
        .replace(/\s+/g, ' ')
        .replace(/;\s*}/g, ';}')
        .trim();
};
/**
 * Create files for serverless functions
 *
 * @return {Promise<void>}
 */
const createServerlessFiles = async (args) => {
    try {
        /* Args */
        const { configExport = 'config', configFile = 'lib/config/config.js', dataExport = dataSource_js_1.dataSource.isContentful() ? 'getAllContentfulData' : 'getAllWordPressData', dataFile = 'lib/serverless/serverlessData.js', previewExport = 'Preview', previewFile = 'lib/serverless/serverlessPreview.js', reloadExport = 'Reload', reloadFile = 'lib/serverless/serverlessReload.js', ajaxExport = 'Ajax', ajaxFile = 'lib/serverless/serverlessAjax.js' } = (0, object_js_1.isObjectStrict)(args) ? args : {};
        /* Serverless folder */
        const serverlessDir = config_js_1.config.serverlessDir;
        if (!(0, string_js_1.isStringStrict)(serverlessDir)) {
            throw new Error('No serverlessDir in config');
        }
        await (0, promises_1.mkdir)((0, node_path_1.resolve)(serverlessDir), { recursive: true });
        /* Ajax file */
        const ajaxServerlessFile = config_js_1.config.serverlessFiles.ajax;
        if ((0, string_js_1.isStringStrict)(ajaxServerlessFile)) {
            const pathDepth = (0, path_js_1.getPathDepth)(`${serverlessDir}/${ajaxServerlessFile}`);
            const content = `
        import { ${configExport} } from '${pathDepth}${configFile}';
        import { ${ajaxExport} } from '${pathDepth}${ajaxFile}';
        const render = async ({ request, functionPath, env }) => {
          return await Ajax({
            request,
            functionPath,
            env,
            siteConfig: ${configExport}
          })
        };
        export const onRequestPost = [render];
      `;
            const path = (0, node_path_1.resolve)(serverlessDir, ajaxServerlessFile);
            const dir = (0, node_path_1.dirname)(path);
            await (0, promises_1.mkdir)((0, node_path_1.resolve)(serverlessDir, dir), { recursive: true });
            await (0, promises_1.writeFile)(path, minify(content));
            (0, print_js_1.print)('[SSF] Successfully wrote', path, 'success');
        }
        /* Preview file */
        const previewServerlessFile = config_js_1.config.serverlessFiles.preview;
        if (config_js_1.config.env.dev && (0, string_js_1.isStringStrict)(previewServerlessFile)) {
            const pathDepth = (0, path_js_1.getPathDepth)(`${serverlessDir}/${previewServerlessFile}`);
            const content = `
        import { ${configExport} } from '${pathDepth}${configFile}';
        import { ${dataExport} } from '${pathDepth}${dataFile}';
        import { ${previewExport} } from '${pathDepth}${previewFile}';
        const render = async ({ request, functionPath, next, env }) => {
          return await ${previewExport}({
            request,
            functionPath,
            next,
            env,
            siteConfig: ${configExport},
            getData: ${dataExport}
          })
        };
        export const onRequestGet = [render];
      `;
            const path = (0, node_path_1.resolve)(serverlessDir, previewServerlessFile);
            const dir = (0, node_path_1.dirname)(path);
            await (0, promises_1.mkdir)((0, node_path_1.resolve)(serverlessDir, dir), { recursive: true });
            await (0, promises_1.writeFile)(path, minify(content));
            (0, print_js_1.print)('[SSF] Successfully wrote', path, 'success');
        }
        /* Routes */
        const routes = Object.keys(config_js_1.config.serverlessRoutes);
        if (routes.length > 0) {
            const reloadServerlessFile = config_js_1.config.serverlessFiles.reload;
            for (const type of routes) {
                const routesArr = config_js_1.config.serverlessRoutes[type];
                if (!(0, array_js_1.isArrayStrict)(routesArr)) {
                    continue;
                }
                for (const route of routesArr) {
                    let { path = '', content = '' } = route;
                    if (type === 'reload' && (0, string_js_1.isStringStrict)(reloadServerlessFile) && (0, string_js_1.isStringStrict)(path)) {
                        path = `${path}/${reloadServerlessFile}`;
                        const pathDepth = (0, path_js_1.getPathDepth)(`${serverlessDir}/${path}`);
                        content = `
              import { ${configExport} } from '${pathDepth}${configFile}';
              import { ${dataExport} } from '${pathDepth}${dataFile}';
              import { ${reloadExport} } from '${pathDepth}${reloadFile}';
              const render = async ({ request, functionPath, next, env }) => {
                return await ${reloadExport}({
                  request,
                  functionPath,
                  next,
                  env,
                  siteConfig: ${configExport},
                  getData: ${dataExport}
                })
              };
              export const onRequestGet = [render];
            `;
                    }
                    if ((0, string_js_1.isStringStrict)(path) && (0, string_js_1.isStringStrict)(content)) {
                        path = (0, node_path_1.resolve)(serverlessDir, path);
                        const dir = (0, node_path_1.dirname)(path);
                        await (0, promises_1.mkdir)((0, node_path_1.resolve)(serverlessDir, dir), { recursive: true });
                        await (0, promises_1.writeFile)(path, content);
                        (0, print_js_1.print)('[SSF] Successfully wrote', path, 'success');
                    }
                }
            }
        }
    }
    catch (error) {
        (0, print_js_1.print)('[SSF] Error writing serverless files', error);
    }
};
exports.createServerlessFiles = createServerlessFiles;
/**
 * Create redirects file from config redirects array
 *
 * @return {Promise<void>}
 */
const createRedirectsFile = async () => {
    try {
        const redirects = config_js_1.config.redirects.data;
        const path = (0, node_path_1.resolve)(config_js_1.config.redirects.file);
        let redirectsData = '';
        if (redirects.length > 0) {
            redirects.forEach((r) => {
                redirectsData += `${r}\n`;
            });
        }
        await (0, promises_1.writeFile)(path, redirectsData);
        (0, print_js_1.print)('[SSF] Successfully wrote', path, 'success');
    }
    catch (error) {
        (0, print_js_1.print)('[SSF] Error writing redirects file', error);
    }
};
exports.createRedirectsFile = createRedirectsFile;
/**
 * Create files from config store object
 *
 * @return {Promise<void>}
 */
const createStoreFiles = async () => {
    try {
        const files = config_js_1.config.storeFiles;
        await (0, promises_1.mkdir)((0, node_path_1.resolve)(config_js_1.config.storeDir), { recursive: true });
        for (const [, file] of Object.entries(files)) {
            const path = (0, node_path_1.resolve)(config_js_1.config.storeDir, file.name);
            await (0, promises_1.writeFile)(path, file.data);
            (0, print_js_1.print)('[SSF] Successfully wrote', path, 'success');
        }
    }
    catch (error) {
        (0, print_js_1.print)('[SSF] Error writing store files', error);
    }
};
exports.createStoreFiles = createStoreFiles;
