"use strict";
/**
 * Utils - Link
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPermalink = exports.getLink = exports.getSlug = void 0;
const config_js_1 = require("../../config/config.js");
const object_js_1 = require("../object/object.js");
const string_js_1 = require("../string/string.js");
const filter_js_1 = require("../filter/filter.js");
const archive_js_1 = require("../archive/archive.js");
/**
 * Recurse to get ascendents
 *
 * @private
 * @param {string} id
 * @param {ConfigParent[]} parents
 * @return {void}
 */
const getParentSlug = (id = '', parents = []) => {
    if (config_js_1.config.parents[id] !== undefined && id !== '') {
        parents.unshift(config_js_1.config.parents[id]);
        getParentSlug(config_js_1.config.parents[id].id, parents);
    }
};
/**
 * Get slug with archive/taxonomy base and parents
 *
 * @param {LinkSlugArgs} args
 * @return {LinkSlugReturnType}
 */
const getSlug = (args, returnParents = false) => {
    const { id = '', slug = '', page = 0, contentType = 'page', pageData = undefined } = (0, object_js_1.isObjectStrict)(args) ? args : {};
    /* Index */
    const isIndex = slug === 'index';
    if (isIndex && returnParents === false) {
        return '';
    }
    /* Parts */
    let parts = [];
    /* Term/taxonomy */
    const taxonomyInfo = (0, archive_js_1.getTaxonomyInfo)(contentType, pageData);
    const { id: taxonomyId, slug: taxonomySlug, title: taxonomyTitle, contentType: taxonomyType, useContentTypeSlug: taxonomyUseTypeSlug, isPage: taxonomyIsPage } = taxonomyInfo;
    /* Archive */
    const archiveType = (0, string_js_1.isStringStrict)(taxonomyType) ? taxonomyType : contentType;
    const archiveInfo = (0, archive_js_1.getArchiveInfo)(archiveType);
    const { id: archiveId, slug: archiveSlug, title: archiveTitle } = archiveInfo;
    /* Archive and/or taxonomy parent */
    let archiveParent;
    if (archiveSlug !== '' && archiveId !== '') {
        parts.push(archiveSlug);
        archiveParent = {
            contentType: 'page',
            title: archiveTitle,
            slug: archiveSlug,
            id: archiveId
        };
    }
    if (contentType === 'taxonomy' && !taxonomyUseTypeSlug) {
        parts = [];
        archiveParent = undefined;
    }
    if (contentType === 'term' && taxonomySlug !== '' && taxonomyId !== '') {
        if (taxonomyUseTypeSlug) {
            parts.push(taxonomySlug);
        }
        else {
            parts = [taxonomySlug];
        }
        if (taxonomyIsPage) {
            archiveParent = {
                contentType: 'taxonomy',
                title: taxonomyTitle,
                slug: taxonomySlug,
                id: taxonomyId
            };
        }
    }
    /* Content type */
    if (config_js_1.config.typesInSlug.includes(contentType)) {
        parts = [contentType];
    }
    /* Page parents */
    const parents = [];
    getParentSlug(contentType === 'page' ? id : archiveId, parents);
    if (parents.length > 0) {
        parts.push(`${parents.map(({ slug }) => slug).join('/')}`);
    }
    /* Locale */
    const locale = pageData?.locale;
    if ((0, string_js_1.isStringStrict)(locale) && config_js_1.config.localesInSlug.includes(locale)) {
        parts.unshift(locale);
    }
    /* Filter parts */
    parts = (0, filter_js_1.applyFiltersSync)('slugParts', parts, args);
    /* Slug */
    if ((0, string_js_1.isStringStrict)(slug) && !isIndex) {
        parts.push(slug);
    }
    const s = `${parts.length > 0 ? parts.join('/') : ''}${page !== 0 ? `/?page=${page}` : ''}`;
    /* Parents and slug return */
    if (returnParents === true) {
        if (archiveParent !== undefined) {
            parents.unshift(archiveParent);
        }
        const res = {
            slug: s,
            parents
        };
        return res;
    }
    /* Slug return */
    return s;
};
exports.getSlug = getSlug;
/**
 * Get absolute or relative url
 *
 * @param {string} slug
 * @param {boolean} trailingSlash
 * @return {string}
 */
const getPermalink = (slug = '', trailingSlash = true) => {
    let url = '/';
    if (config_js_1.config.env.prod) {
        url = config_js_1.config.env.prodUrl;
    }
    if (slug === '/') {
        return url;
    }
    return `${url}${slug}${slug !== '' && trailingSlash ? '/' : ''}`;
};
exports.getPermalink = getPermalink;
/**
 * Get permalink from external or internal source
 *
 * @param {InternalLink} [internalLink]
 * @param {string} [externalLink]
 * @return {string}
 */
const getLink = (internalLink, externalLink) => {
    if ((0, object_js_1.isObjectStrict)(internalLink)) {
        const slug = internalLink.slug;
        const res = getSlug({
            id: internalLink.id,
            contentType: internalLink.contentType,
            pageData: internalLink,
            slug: (0, string_js_1.isString)(slug) ? slug : ''
        }, false);
        return getPermalink(res);
    }
    return (0, string_js_1.isStringStrict)(externalLink) ? externalLink : '';
};
exports.getLink = getLink;
