"use strict";
/**
 * Utils - Shortcode
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.stripShortcodes = exports.setShortcodes = exports.resetShortcodes = exports.doShortcodes = exports.removeShortcode = exports.addShortcode = void 0;
const object_js_1 = require("../object/object.js");
const array_js_1 = require("../array/array.js");
const string_js_1 = require("../string/string.js");
const function_js_1 = require("../function/function.js");
const number_js_1 = require("../number/number.js");
const escape_js_1 = require("../escape/escape.js");
/**
 * Store shortcode callbacks by name
 *
 * @type {Shortcodes}
 */
let shortcodes = {};
/**
 * Regex for searching x="" in strings
 *
 * @private
 * @type {RegExp}
 */
const attrReg = /[\w-]+=".*?"/g;
/**
 * Extract attributes and inner content from tagged strings
 *
 * @private
 * @param {string} content
 * @param {string} tagNames
 * @param {Shortcode} [props]
 * @return {ShortcodeData[]}
 */
const getShortcodeData = (content, tagNames, props) => {
    /* Content and tag names required */
    if (!(0, string_js_1.isStringStrict)(content) || !(0, string_js_1.isStringStrict)(tagNames)) {
        return [];
    }
    /* Search tags in content */
    const reg = new RegExp(String.raw `\[\/?(?<name>${tagNames})[^\]]*?\]`, 'g');
    const matches = [...content.matchAll(reg)];
    if (matches.length === 0) {
        return [];
    }
    /* Store data items */
    const data = [];
    /* Recurse matches */
    matches.forEach((opening) => {
        /* Name required */
        const name = opening?.groups?.name;
        if (!(0, string_js_1.isStringStrict)(name)) {
            return;
        }
        /* Skip closing tag */
        const tag = opening[0];
        if (tag.slice(0, 2) === '[/') {
            return;
        }
        /* Corresponding closing tag required */
        const closingTag = `[/${name}]`;
        const closingIndex = matches.findIndex((m) => m[0] === closingTag);
        if (closingIndex === -1) {
            return;
        }
        const closingArr = matches.splice(closingIndex, 1);
        const closing = closingArr[0];
        /* Shortcode info */
        const info = props === undefined ? shortcodes[name] : props;
        if (info === undefined) {
            return;
        }
        /* Attributes from opening tag */
        const attributes = {};
        const attributeTypes = (0, object_js_1.isObjectStrict)(info.attributeTypes) ? info.attributeTypes : {};
        const attr = tag.match(attrReg);
        if ((0, array_js_1.isArrayStrict)(attr)) {
            attr.forEach((a) => {
                const [key, value] = a.split('=');
                if (key === undefined || value === undefined) {
                    return;
                }
                let val = (0, escape_js_1.escape)(value.replace(/"/g, ''));
                if ((0, string_js_1.isStringStrict)(attributeTypes[key])) {
                    const type = attributeTypes[key];
                    if (type === 'number') {
                        const num = parseInt(val, 10);
                        val = isNaN(num) ? 0 : num;
                    }
                    if (type === 'boolean') {
                        val = val === 'true';
                    }
                }
                attributes[key] = val;
            });
        }
        /* Content including and excluding tags */
        let replaceContent = '';
        let innerContent = '';
        const startIndex = opening.index;
        const endIndex = closing?.index;
        if ((0, number_js_1.isNumber)(startIndex) && (0, number_js_1.isNumber)(endIndex)) {
            replaceContent = content.slice(startIndex, endIndex + closingTag.length);
            innerContent = content.slice(startIndex + tag.length, endIndex);
        }
        /* Check for children */
        let children = [];
        const child = info.child;
        if ((0, object_js_1.isObjectStrict)(child)) {
            children = getShortcodeData(innerContent, child.name, child);
        }
        /* Add data */
        data.push({
            name,
            replaceContent,
            content: innerContent,
            attributes,
            children
        });
    });
    /* Output */
    return data;
};
/**
 * Add shortcode to shortcodes object
 *
 * @param {string} name
 * @param {Shortcode} shortcode
 * @return {boolean}
 */
const addShortcode = (name, shortcode) => {
    if (!(0, string_js_1.isStringStrict)(name) || !(0, object_js_1.isObjectStrict)(shortcode)) {
        return false;
    }
    shortcodes[name] = shortcode;
    return true;
};
exports.addShortcode = addShortcode;
/**
 * Remove shortcode from shortcodes object
 *
 * @param {string} name
 * @return {boolean}
 */
const removeShortcode = (name) => {
    if (!(0, string_js_1.isStringStrict)(name)) {
        return false;
    }
    if (shortcodes[name] === undefined) {
        return false;
    }
    // @ts-expect-error: Type 'undefined' is not assignable to type 'Shortcode'
    shortcodes[name] = undefined;
    return true;
};
exports.removeShortcode = removeShortcode;
/**
 * Transform content string with shortcode callbacks
 *
 * @param {string} content
 * @return {Promise<string>}
 */
const doShortcodes = async (content) => {
    /* Check if any shortcodes */
    const names = Object.keys(shortcodes);
    if (names.length === 0) {
        return content;
    }
    /* Get data */
    const data = getShortcodeData(content, names.join('|'));
    if (data.length === 0) {
        return content;
    }
    /* Replace with shortcode content */
    let newContent = content;
    for (const d of data) {
        const { name, replaceContent } = d;
        const callback = shortcodes?.[name]?.callback;
        if ((0, function_js_1.isFunction)(callback)) {
            const res = await callback(d);
            if ((0, string_js_1.isStringStrict)(res)) {
                newContent = newContent.replace(replaceContent, res);
            }
        }
    }
    return newContent;
};
exports.doShortcodes = doShortcodes;
/**
 * Empty shortcodes object
 *
 * @return {void}
 */
const resetShortcodes = () => {
    shortcodes = {};
};
exports.resetShortcodes = resetShortcodes;
/**
 * Fill shortcodes object
 *
 * @param {Shortcodes} args
 * @return {boolean}
 */
const setShortcodes = (args) => {
    if (!(0, object_js_1.isObjectStrict)(args)) {
        return false;
    }
    const names = Object.keys(args);
    if (names.length === 0) {
        return false;
    }
    resetShortcodes();
    names.forEach((n) => {
        const shortcode = args[n];
        if (shortcode === undefined) {
            return;
        }
        addShortcode(n, shortcode);
    });
    return true;
};
exports.setShortcodes = setShortcodes;
/**
 * Remove shortcodes from string
 *
 * @param {string} content
 * @return {string}
 */
const stripShortcodes = (content) => {
    /* Check if any shortcodes */
    const names = Object.keys(shortcodes);
    if (names.length === 0) {
        return content;
    }
    /* Replace tags with empty strings */
    return content.replace(String.raw `/\[\/?(?:${names.join('|')})[^\]]*?\]/g`, '');
};
exports.stripShortcodes = stripShortcodes;
