"use strict";
/**
 * Utils - WordPress Data
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllWordPressData = exports.getWordPressData = void 0;
const filter_js_1 = require("../filter/filter.js");
const object_js_1 = require("../object/object.js");
const array_js_1 = require("../array/array.js");
const string_js_1 = require("../string/string.js");
const function_js_1 = require("../function/function.js");
const number_js_1 = require("../number/number.js");
const json_js_1 = require("../json/json.js");
const config_js_1 = require("../../config/config.js");
const print_js_1 = require("../print/print.js");
const wordpressDataNormal_js_1 = require("./wordpressDataNormal.js");
/**
 * Normalize route by type
 *
 * @private
 * @param {string} type
 * @return {string}
 */
const getRoute = (type) => {
    const routes = {
        page: 'pages',
        post: 'posts',
        taxonomy: 'taxonomies',
        category: 'categories',
        tag: 'tags',
        attachment: 'media',
        navigation: 'menus',
        navigationItem: 'menu-items'
    };
    if (routes[type] !== undefined) {
        return routes[type];
    }
    return type;
};
/**
 * Fetch data from wordpress cms or cache
 *
 * @param {string} key
 * @param {string} route
 * @param {WordPressDataParams} [params]
 * @param {number} [_page]
 * @return {Promise<RenderItem[]>}
 */
const getWordPressData = async (key, route, params = {}, _page = 1) => {
    try {
        /* Key required for cache */
        if (!(0, string_js_1.isStringStrict)(key)) {
            throw new Error('No key');
        }
        /* Check cache */
        if (config_js_1.config.env.cache) {
            let cacheData = [];
            const cacheDataFilterArgs = {
                key,
                type: 'get'
            };
            cacheData = await (0, filter_js_1.applyFilters)('cacheData', cacheData, cacheDataFilterArgs);
            if ((0, object_js_1.isObject)(cacheData)) {
                return structuredClone(cacheData);
            }
        }
        /* Route required */
        if (!(0, string_js_1.isStringStrict)(route)) {
            throw new Error('No route');
        }
        /* Credentials */
        const { devUser, prodUser, devCredential, prodCredential, devHost, prodHost, ssl = true } = config_js_1.config.cms;
        let user = devUser;
        let pass = devCredential;
        let host = devHost;
        let status = 'any';
        if (config_js_1.config.env.prod) {
            user = prodUser;
            pass = prodCredential;
            host = prodHost;
            status = 'publish';
        }
        if (!(0, string_js_1.isStringStrict)(user) || !(0, string_js_1.isStringStrict)(pass) || !(0, string_js_1.isStringStrict)(host)) {
            throw new Error('No credentials');
        }
        /* Params */
        let url = `${ssl ? 'https' : 'http'}://${host}/wp-json/wp/v2/${route}?_embed&status=${status}`;
        let loop = false;
        params = (0, object_js_1.isObjectStrict)(params) ? params : {};
        for (const [key, value] of Object.entries(params)) {
            let val = value;
            if (key === 'per_page' && value === -1) {
                val = 100;
                loop = true;
            }
            url += `&${key}=${val.toString()}`;
        }
        if (loop) {
            url += `&page=${_page}`;
        }
        /* Check and transform data */
        const resp = await fetch(url, {
            headers: {
                Authorization: `Basic ${btoa(`${user}:${pass}`)}`
            }
        });
        const data = await resp.json();
        const isObj = (0, object_js_1.isObjectStrict)(data);
        /* Check if error */
        const isErr = isObj && data.message !== undefined;
        if (!resp.ok || isErr) {
            const message = isObj ? data.message : '';
            throw new Error((0, string_js_1.isStringStrict)(message) ? message : 'Bad request', {
                cause: data
            });
        }
        /* Total */
        const total = resp.headers.get('X-WP-TotalPages');
        const totalNum = (0, string_js_1.isStringStrict)(total) ? parseInt(total, 10) : 1;
        const totalPages = (0, number_js_1.isNumber)(totalNum) ? totalNum : 1;
        /* Normalize */
        const dataItems = (0, array_js_1.isArray)(data) ? data : [data];
        let newData = (0, wordpressDataNormal_js_1.normalizeWordPressData)(dataItems);
        if (loop && _page < totalPages) {
            const pagData = await getWordPressData(key, route, {
                per_page: -1
            }, _page + 1);
            if ((0, array_js_1.isArray)(pagData)) {
                newData = [
                    ...newData,
                    ...pagData
                ];
            }
        }
        /* Store in cache */
        if (config_js_1.config.env.cache) {
            const cacheDataFilterArgs = {
                key,
                type: 'set',
                data
            };
            await (0, filter_js_1.applyFilters)('cacheData', newData, cacheDataFilterArgs);
        }
        /* Output */
        return newData;
    }
    catch (error) {
        (0, print_js_1.print)('[SSF] Error fetching WordPress data', error);
        /* Store in cache (avoid extra calls when no result) */
        if (config_js_1.config.env.cache) {
            const cacheDataFilterArgs = {
                key,
                type: 'set',
                data: []
            };
            await (0, filter_js_1.applyFilters)('cacheData', [], cacheDataFilterArgs);
        }
        /* Output */
        return [];
    }
};
exports.getWordPressData = getWordPressData;
/**
 * Fetch data from all content types or single entry if serverless
 *
 * @param {AllWordPressDataArgs} args
 * @return {Promise<RenderAllData|undefined>}
 */
const getAllWordPressData = async (args) => {
    try {
        /* Args */
        const argsObj = (0, object_js_1.isObjectStrict)(args) ? args : {};
        const { serverlessData, previewData, filterData, filterAllData } = argsObj;
        /* Store all data */
        let allData = {
            navigation: [],
            navigationItem: [],
            redirect: [],
            content: {
                page: []
            }
        };
        /* Serverless */
        const isServerless = serverlessData !== undefined;
        const isPreview = previewData !== undefined;
        /* Get single entry data if serverless or preview data */
        let isEntry = false;
        if (isServerless || isPreview) {
            let contentType = '';
            let id = '';
            if (isServerless) {
                const slugsData = await (0, json_js_1.getJsonFile)('slugs');
                const path = serverlessData.path;
                if (slugsData !== undefined) {
                    const item = slugsData[path];
                    if ((0, object_js_1.isObjectStrict)(item)) {
                        id = (0, string_js_1.isStringStrict)(item.id) ? item.id : '';
                        contentType = (0, string_js_1.isStringStrict)(item.contentType) ? item.contentType : '';
                    }
                }
            }
            if (isPreview) {
                id = previewData.id;
                contentType = previewData.contentType;
            }
            if (id !== '') {
                const key = `serverless_${id}_${contentType}`;
                const data = await getWordPressData(key, `${getRoute(contentType)}/${id}`);
                if ((0, array_js_1.isArray)(data)) {
                    isEntry = true;
                    allData.content[contentType] = data;
                }
            }
        }
        /* Get partial data - not serverless */
        if (!isServerless || !isEntry) {
            const partial = config_js_1.config.partialTypes;
            for (const contentType of partial) {
                allData[contentType] = [];
                const key = `all_${contentType}`;
                let data = await getWordPressData(key, getRoute(contentType), {
                    per_page: -1
                });
                if ((0, function_js_1.isFunction)(filterData)) {
                    data = filterData(data, serverlessData, previewData);
                }
                if ((0, array_js_1.isArray)(data)) {
                    allData[contentType] = data;
                }
            }
        }
        /* Get whole data (for page generation) - not serverless or preview */
        if ((!isServerless && !isPreview) || !isEntry) {
            const whole = config_js_1.config.wholeTypes;
            for (const contentType of whole) {
                allData.content[contentType] = [];
                const key = `all_${contentType}`;
                let data = await getWordPressData(key, getRoute(contentType), {
                    per_page: -1
                });
                if ((0, function_js_1.isFunction)(filterData)) {
                    data = filterData(data, serverlessData, previewData);
                }
                if ((0, array_js_1.isArray)(data)) {
                    allData.content[contentType] = data;
                }
            }
        }
        /* Filter all data */
        if ((0, function_js_1.isFunction)(filterAllData)) {
            allData = filterAllData(allData, serverlessData, previewData);
        }
        /* Output */
        return allData;
    }
    catch (error) {
        (0, print_js_1.print)('[SSF] Error getting all WordPress data', error);
    }
};
exports.getAllWordPressData = getAllWordPressData;
