/**
 * Utils - Link
 */
import { config } from '../../config/config.js';
import { isObjectStrict } from '../object/object.js';
import { isString, isStringStrict } from '../string/string.js';
import { applyFiltersSync } from '../filter/filter.js';
import { getArchiveInfo, getTaxonomyInfo } from '../archive/archive.js';
/**
 * Recurse to get ascendents
 *
 * @private
 * @param {string} id
 * @param {ConfigParent[]} parents
 * @return {void}
 */
const getParentSlug = (id = '', parents = []) => {
    if (config.parents[id] !== undefined && id !== '') {
        parents.unshift(config.parents[id]);
        getParentSlug(config.parents[id].id, parents);
    }
};
/**
 * Get slug with archive/taxonomy base and parents
 *
 * @param {LinkSlugArgs} args
 * @return {LinkSlugReturnType}
 */
const getSlug = (args, returnParents = false) => {
    const { id = '', slug = '', page = 0, contentType = 'page', pageData = undefined } = isObjectStrict(args) ? args : {};
    /* Index */
    const isIndex = slug === 'index';
    if (isIndex && returnParents === false) {
        return '';
    }
    /* Parts */
    let parts = [];
    /* Term/taxonomy */
    const taxonomyInfo = getTaxonomyInfo(contentType, pageData);
    const { id: taxonomyId, slug: taxonomySlug, title: taxonomyTitle, contentType: taxonomyType, useContentTypeSlug: taxonomyUseTypeSlug, isPage: taxonomyIsPage } = taxonomyInfo;
    /* Archive */
    const archiveType = isStringStrict(taxonomyType) ? taxonomyType : contentType;
    const archiveInfo = getArchiveInfo(archiveType);
    const { id: archiveId, slug: archiveSlug, title: archiveTitle } = archiveInfo;
    /* Archive and/or taxonomy parent */
    let archiveParent;
    if (archiveSlug !== '' && archiveId !== '') {
        parts.push(archiveSlug);
        archiveParent = {
            contentType: 'page',
            title: archiveTitle,
            slug: archiveSlug,
            id: archiveId
        };
    }
    if (contentType === 'taxonomy' && !taxonomyUseTypeSlug) {
        parts = [];
        archiveParent = undefined;
    }
    if (contentType === 'term' && taxonomySlug !== '' && taxonomyId !== '') {
        if (taxonomyUseTypeSlug) {
            parts.push(taxonomySlug);
        }
        else {
            parts = [taxonomySlug];
        }
        if (taxonomyIsPage) {
            archiveParent = {
                contentType: 'taxonomy',
                title: taxonomyTitle,
                slug: taxonomySlug,
                id: taxonomyId
            };
        }
    }
    /* Content type */
    if (config.typesInSlug.includes(contentType)) {
        parts = [contentType];
    }
    /* Page parents */
    const parents = [];
    getParentSlug(contentType === 'page' ? id : archiveId, parents);
    if (parents.length > 0) {
        parts.push(`${parents.map(({ slug }) => slug).join('/')}`);
    }
    /* Locale */
    const locale = pageData?.locale;
    if (isStringStrict(locale) && config.localesInSlug.includes(locale)) {
        parts.unshift(locale);
    }
    /* Filter parts */
    parts = applyFiltersSync('slugParts', parts, args);
    /* Slug */
    if (isStringStrict(slug) && !isIndex) {
        parts.push(slug);
    }
    const s = `${parts.length > 0 ? parts.join('/') : ''}${page !== 0 ? `/?page=${page}` : ''}`;
    /* Parents and slug return */
    if (returnParents === true) {
        if (archiveParent !== undefined) {
            parents.unshift(archiveParent);
        }
        const res = {
            slug: s,
            parents
        };
        return res;
    }
    /* Slug return */
    return s;
};
/**
 * Get absolute or relative url
 *
 * @param {string} slug
 * @param {boolean} trailingSlash
 * @return {string}
 */
const getPermalink = (slug = '', trailingSlash = true) => {
    let url = '/';
    if (config.env.prod) {
        url = config.env.prodUrl;
    }
    if (slug === '/') {
        return url;
    }
    return `${url}${slug}${slug !== '' && trailingSlash ? '/' : ''}`;
};
/**
 * Get permalink from external or internal source
 *
 * @param {InternalLink} [internalLink]
 * @param {string} [externalLink]
 * @return {string}
 */
const getLink = (internalLink, externalLink) => {
    if (isObjectStrict(internalLink)) {
        const slug = internalLink.slug;
        const res = getSlug({
            id: internalLink.id,
            contentType: internalLink.contentType,
            pageData: internalLink,
            slug: isString(slug) ? slug : ''
        }, false);
        return getPermalink(res);
    }
    return isStringStrict(externalLink) ? externalLink : '';
};
/* Exports */
export { getSlug, getLink, getPermalink };
